export declare const ip: {
    address(): string;
    isLoopback(addr: string): boolean;
    loopback(family: IpFamily): string;
};
type IpFamily = 'ipv4' | 'ipv6';
export {};
//# sourceMappingURL=ip.d.ts.map